 /*  matmath.c

     This is a roughly self-contained code module for matrix
     math, which started with the Numerical Recipes in C code
     and matrix representations.

     J. Watlington, 11/15/95

     Modified:
 */

 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include "my_math.h"

 #define NR_END 1
 #define FREE_ARG char*

 /**********************************************************

      Vector Math Functions

 */

 void vec_copy( double *src, double *dst, int nudoubleents )
 {
   int  i;

   for( i = 1; i <= nudoubleents; i++ )
     dst[ i ] = src[ i ];
 }

 void vec_add( double *a, double *b, double *c, int n )
 {
   int i;
   double  *a_ptr, *b_ptr, *c_ptr;

   a_ptr = a + 1;
   b_ptr = b + 1;
   c_ptr = c + 1;

   for( i = 1; i <= n; i++)
     *c_ptr++ = *a_ptr++ + *b_ptr++;
 }

 /* vec_sub()
    This function computes C = A - B, for vectors of size n  */

 void vec_sub( double *a, double *b, double *c, int n )
 {
   int i;
   double  *a_ptr, *b_ptr, *c_ptr;

   a_ptr = a + 1;
   b_ptr = b + 1;
   c_ptr = c + 1;

   for( i = 1; i <= n; i++)
     *c_ptr++ = *a_ptr++ - *b_ptr++;
 }


 /**********************************************************

   Matrix math functions

 */

 void mat_add( double **a, double **b, double **c, int m, int n )
 {
   int i,j;
   double  *a_ptr, *b_ptr, *c_ptr;

   for( j = 1; j <= m; j++)
     {
       a_ptr = &a[j][1];
       b_ptr = &b[j][1];
       c_ptr = &c[j][1];

       for( i = 1; i <= n; i++)
	 *c_ptr++ = *a_ptr++ + *b_ptr++;
     }
 }


 /* mat_sub()
    This function computes C = A - B, for matrices of size m x n  */

 void mat_sub( double **a, double **b, double **c, int m, int n )
 {
   int i,j;
   double  *a_ptr, *b_ptr, *c_ptr;

   for( j = 1; j <= m; j++)
     {
       a_ptr = &a[j][1];
       b_ptr = &b[j][1];
       c_ptr = &c[j][1];

       for( i = 1; i <= n; i++)
	 *c_ptr++ = *a_ptr++ - *b_ptr++;
     }
 }

 /*  mat_mult
     This function performs a matrix multiplication.
 */
 void mat_mult( double **a, double **b, double **c,
	       int a_rows, int a_cols, int b_cols )
 {
   int i, j, k;
   double  *a_ptr;
   double  temp;

   for( i = 1; i <= a_rows; i++)
     {
       a_ptr = &a[i][0];
       for( j = 1; j <= b_cols; j++ )
	 {
	   temp = 0.0;
	   for( k = 1; k <= a_cols; k++ )
	     temp = temp + (a_ptr[k] * b[k][j]); 
	   c[i][j] = temp;
	 }
     }
 }


 /*  mat_mult_vector
     This function performs a matrix x vector multiplication.
 */
 void mat_mult_vector( double **a, double *b, double *c,
	       int a_rows, int a_cols )
 {
   int i, j, k;
   double  *a_ptr, *b_ptr;
   double  temp;

   for( i = 1; i <= a_rows; i++)
     {
       a_ptr = &a[i][0];
       b_ptr = &b[1];
       temp = 0.0;

       for( k = 1; k <= a_cols; k++ )
	 temp += a_ptr[ k ] * *b_ptr++;

       c[i] = temp;
     }
 }


 /*  mat_mult_transpose
     This function performs a matrix multiplication of A x transpose B.
 */
 void mat_mult_transpose( double **a, double **b, double **c,
	       int a_rows, int a_cols, int b_cols )
 {
   int i, j, k;
   double  *a_ptr, *b_ptr;
   double  temp;

   for( i = 1; i <= a_rows; i++)
     {
       a_ptr = &a[i][0];
       for( j = 1; j <= b_cols; j++ )
	 {
	   b_ptr = &b[j][1];

	   temp = (double)0;

	   for( k = 1; k <= a_cols; k++ )
	     temp += a_ptr[ k ] * *b_ptr++;

	   c[i][j] = temp;
	 }
     }
 }

 /*  mat_transpose_mult
     This function performs a matrix multiplication of transpose A x B.
     a_rows refers to the transposed A, is a_cols in actual A storage
     a_cols is same, is a_rows in actual A storage
 */
 void mat_transpose_mult( double **A, double **B, double **C,
	       int a_rows, int a_cols, int b_cols )
 {
   int i, j, k;
   double  temp;

   for( i = 1; i <= a_rows; i++)
     {
       for( j = 1; j <= b_cols; j++ )
	 {
	   temp = 0.0;

	   for( k = 1; k <= a_cols; k++ )
	     temp += A[ k ][ i ] * B[ k ][ j ];

	   C[ i ][ j ] = temp;
	 }
     }
 }

 void mat_copy( double **src, double **dst, int num_rows, int num_cols )
 {
   int  i, j;

   for( i = 1; i <= num_rows; i++ )
     for( j = 1; j <= num_cols; j++ )
       dst[ i ][ j ] = src[ i ][ j ];
 }


 /* gaussj()
    This function performs gauss-jordan elimination to solve a set
    of linear equations, at the same time generating the inverse of
    the A matrix.

    (C) Copr. 1986-92 Numerical Recipes Software `2$m.1.9-153.
 */

 #define SWAP(a,b) {temp=(a);(a)=(b);(b)=temp;}

 void gaussj(double **a, int n, double **b, int m)
 {
   int *indxc,*indxr,*ipiv;
   int i,icol,irow,j,k,l,ll;
   double big,dum,pivinv,temp;

   indxc=ivector(1,n);
   indxr=ivector(1,n);
   ipiv=ivector(1,n);
   for (j=1;j<=n;j++) ipiv[j]=0;
   for (i=1;i<=n;i++) {
     big=0.0;
     for (j=1;j<=n;j++)
       if (ipiv[j] != 1)
	 for (k=1;k<=n;k++) {
	   if (ipiv[k] == 0) {
	     if (fabs(a[j][k]) >= big) {
	       big=fabs(a[j][k]);
	       irow=j;
	       icol=k;
	     }
	   } else if (ipiv[k] > 1) printf("gaussj: Singular Matrix-1");
	 }
     ++(ipiv[icol]);
     if (irow != icol) {
       for (l=1;l<=n;l++) SWAP(a[irow][l],a[icol][l])
       for (l=1;l<=m;l++) SWAP(b[irow][l],b[icol][l])
     }
     indxr[i]=irow;
     indxc[i]=icol;
     if (a[icol][icol] == 0.0) printf("gaussj: Singular Matrix-2");
     pivinv=1.0/a[icol][icol];
     a[icol][icol]=1.0;
     for (l=1;l<=n;l++) a[icol][l] *= pivinv;
     for (l=1;l<=m;l++) b[icol][l] *= pivinv;
     for (ll=1;ll<=n;ll++)
       if (ll != icol) {
	 dum=a[ll][icol];
	 a[ll][icol]=0.0;
	 for (l=1;l<=n;l++) a[ll][l] -= a[icol][l]*dum;
	 for (l=1;l<=m;l++) b[ll][l] -= b[icol][l]*dum;
       }
   }
   for (l=n;l>=1;l--) {
     if (indxr[l] != indxc[l])
       for (k=1;k<=n;k++)
	 SWAP(a[k][indxr[l]],a[k][indxc[l]]);
   }

   free_ivector(ipiv,1,n);
   free_ivector(indxr,1,n);
   free_ivector(indxc,1,n);
 }
 #undef SWAP


 /**********************************************************

   Quaternion math

   A quaternion is a four vector used to represent rotation in
   three-space.  The representation being used here is from
   Ali Azarbayejani.

   It is stored in components 0 through 3 of the vector.
 */

 double *quaternion( void )
 {
   double  *v;

   v = vector( 0, 3 );
   v[0] = (double)1.0;  v[1] = (double)0.0;
   v[2] = (double)0.0;  v[3] = (double)0.0;
 }

 void quaternion_update( double *quat, double wx,
			double wy, double wz )
 {
   int           i;
   double  temp;
   double  delta[QUATERNION_SIZE], new[QUATERNION_SIZE];

   /*  First, define a quaternion from the angular velocities  */

   temp = wx*wx + wy*wy + wz*wz;
   delta[0] = sqrt( 1 - (temp * 0.25) );
   temp = 1.0 / 2.0;
   delta[1] = wx * temp;
   delta[2] = wy * temp;
   delta[3] = wz * temp;

   /*  Now multiply it with the input quaternion, to update it. */

   new[0] = quat[0]*delta[0] - quat[1]*delta[1] - quat[2]*delta[2]
     - quat[3]*delta[3];
   new[1] = quat[1]*delta[0] + quat[0]*delta[1] - quat[3]*delta[2]
     + quat[2]*delta[3];
   new[2] = quat[2]*delta[0] + quat[3]*delta[1] + quat[0]*delta[2]
     - quat[1]*delta[3];
   new[3] = quat[3]*delta[0] - quat[2]*delta[1] + quat[1]*delta[2]
     + quat[0]*delta[3];

   /*  Finally, normalize the quaternion and copy it back  */

   temp = new[0]*new[0] + new[1]*new[1] + new[2]*new[2] + new[3]*new[3];
   if( temp < MIN_QUATERNION_MAGNITUDE )
     {
       quat[0] = 1.0;
       quat[1] = 0.0; quat[2] = 0.0; quat[3] = 0.0;
       return;
     }

   temp = 1.0 / (double)sqrt( (double)temp );
   if( temp != 1.0 )
     {
       for( i = 0; i < QUATERNION_SIZE; i++ )
	 quat[i] = new[i] * temp;
     }
   else
     for( i = 0; i < QUATERNION_SIZE; i++ )
       quat[i] = new[i];
 }

 /*  quaternion_to_rotation
     This function takes pointers to a quaternion input, and a rotation
     matrix for the results.  It calculates the 3D rotation matrix that
     is equivalent to the quaternion.
 */
 void quaternion_to_rotation( double *quat, double **rot )
 {
   double  q01 = quat[0] * quat[1];
   double  q02 = quat[0] * quat[2];
   double  q03 = quat[0] * quat[3];
   double  q11 = quat[1] * quat[1];
   double  q12 = quat[1] * quat[2];
   double  q13 = quat[1] * quat[3];
   double  q22 = quat[2] * quat[2];
   double  q23 = quat[2] * quat[3];
   double  q33 = quat[3] * quat[3];

   rot[1][1] = 1.0 - 2.0 * (q22 + q33);
   rot[1][2] =     - 2.0 * (q03 - q12);
   rot[1][3] =       2.0 * (q02 + q13);
   rot[2][1] =       2.0 * (q03 + q12);
   rot[2][2] = 1.0 - 2.0 * (q22 + q33);
   rot[2][3] =     - 2.0 * (q01 - q23);
   rot[3][1] =     - 2.0 * (q02 - q13);
   rot[3][2] =       2.0 * (q01 + q23);
   rot[3][3] = 1.0 - 2.0 * (q11 + q22);
 }

 /**********************************************************

   Vector and Matrix allocation and deallocation routines

 */

 void nrerror(char error_text[])
 /* Numerical Recipes standard error handler */
 {
	 printf("Numerical Recipes run-time error...\n");
	 printf("%s\n",error_text);
	 printf("...now exiting to system...\n");
	 exit(1);
 }

 double *vector(long nl, long nh)
 /* allocate a vector with subscript range v[nl..nh] */
 {
	 double *v;

	 v=(double *)malloc((size_t) ((nh-nl+1+NR_END) *
					    sizeof(double)));
	 if (!v) printf("allocation failure in vector()");
	 return v-nl+NR_END;
 }

 float *fvector(long nl, long nh)
 /* allocate a vector with subscript range v[nl..nh] */
 {
	 float *v;

	 v=(float *)malloc((size_t) ((nh-nl+1+NR_END) *
					    sizeof(float)));
	 if (!v) printf("allocation failure in vector()");
	 return v-nl+NR_END;
 }

 int *ivector(long nl, long nh)
 /* allocate an int vector with subscript range v[nl..nh] */
 {
	 int *v;

	 v=(int *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(int)));
	 if (!v) printf("allocation failure in ivector()");
	 return v-nl+NR_END;
 }

 unsigned char *cvector(long nl, long nh)
 /* allocate an unsigned char vector with subscript range v[nl..nh] */
 {
	 unsigned char *v;

	 v=(unsigned char *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(unsigned char)));
	 if (!v) printf("allocation failure in cvector()");
	 return v-nl+NR_END;
 }

 unsigned long *lvector(long nl, long nh)
 /* allocate an unsigned long vector with subscript range v[nl..nh] */
 {
	 unsigned long *v;

	 v=(unsigned long *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(long)));
	 if (!v) printf("allocation failure in lvector()");
	 return v-nl+NR_END;
 }

 double *dvector(long nl, long nh)
 /* allocate a double vector with subscript range v[nl..nh] */
 {
	 double *v;

	 v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double)));
	 if (!v) nrerror("allocation failure in dvector()");
	 return v-nl+NR_END;
 }

 double **matrix(long nrl, long nrh, long ncl, long nch)
 /* allocate a double matrix with subscript range m[nrl..nrh][ncl..nch] */
 {
	 long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	 double **m;

	 /* allocate pointers to rows */
	 m=(double **)malloc((size_t)((nrow+NR_END)
					     *sizeof(double *)));
	 if (!m) nrerror("allocation failure 1 in matrix()");
	 m += NR_END;
	 m -= nrl;

	 /* allocate rows and set pointers to them */
	 m[nrl]=(double *)malloc((size_t)((nrow*ncol+NR_END) 
						*sizeof(double)));
	 if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
	 m[nrl] += NR_END;
	 m[nrl] -= ncl;

	 for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	 /* return pointer to array of pointers to rows */
	 return m;
 }


 float **fmatrix(long nrl, long nrh, long ncl, long nch)
 /* allocate a float matrix with subscript range m[nrl..nrh][ncl..nch] */
 {
	 long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	 float **m;

	 /* allocate pointers to rows */
	 m=(float **) malloc((size_t)((nrow+NR_END)*sizeof(float*)));
	 if (!m) nrerror("allocation failure 1 in matrix()");
	 m += NR_END;
	 m -= nrl;

	 /* allocate rows and set pointers to them */
	 m[nrl]=(float *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(float)));
	 if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
	 m[nrl] += NR_END;
	 m[nrl] -= ncl;

	 for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	 /* return pointer to array of pointers to rows */
	 return m;
 }

 double **dmatrix(long nrl, long nrh, long ncl, long nch)
 /* allocate a double matrix with subscript range m[nrl..nrh][ncl..nch] */
 {
	 long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	 double **m;

	 /* allocate pointers to rows */
	 m=(double **) malloc((size_t)((nrow+NR_END)*sizeof(double*)));
	 if (!m) nrerror("allocation failure 1 in matrix()");
	 m += NR_END;
	 m -= nrl;

	 /* allocate rows and set pointers to them */
	 m[nrl]=(double *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(double)));
	 if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
	 m[nrl] += NR_END;
	 m[nrl] -= ncl;

	 for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	 /* return pointer to array of pointers to rows */
	 return m;
 }

 int **imatrix(long nrl, long nrh, long ncl, long nch)
 /* allocate a int matrix with subscript range m[nrl..nrh][ncl..nch] */
 {
	 long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	 int **m;

	 /* allocate pointers to rows */
	 m=(int **) malloc((size_t)((nrow+NR_END)*sizeof(int*)));
	 if (!m) nrerror("allocation failure 1 in matrix()");
	 m += NR_END;
	 m -= nrl;


	 /* allocate rows and set pointers to them */
	 m[nrl]=(int *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(int)));
	 if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
	 m[nrl] += NR_END;
	 m[nrl] -= ncl;

	 for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	 /* return pointer to array of pointers to rows */
	 return m;
 }

 double **submatrix(double **a,
			  long oldrl, long oldrh, long oldcl, long oldch,
	 long newrl, long newcl)
 /* point a submatrix [newrl..][newcl..] to a[oldrl..oldrh][oldcl..oldch] */
 {
	 long i,j,nrow=oldrh-oldrl+1,ncol=oldcl-newcl;
	 double **m;

	 /* allocate array of pointers to rows */
	 m=(double **) malloc((size_t) ((nrow+NR_END)
					      *sizeof(double*)));
	 if (!m) nrerror("allocation failure in submatrix()");
	 m += NR_END;
	 m -= newrl;

	 /* set pointers to rows */
	 for(i=oldrl,j=newrl;i<=oldrh;i++,j++) m[j]=a[i]+ncol;

	 /* return pointer to array of pointers to rows */
	 return m;
 }

 double **convert_matrix(double *a,
			       long nrl, long nrh, long ncl, long nch)
 /* allocate a double matrix m[nrl..nrh][ncl..nch] that points
 to the matrix declared in the standard C manner as a[nrow][ncol], where
 nrow=nrh-nrl+1 and ncol=nch-ncl+1. The routine should be called with the
 address &a[0][0] as the first argument. */
 {
	 long i,j,nrow=nrh-nrl+1,ncol=nch-ncl+1;
	 double **m;

	 /* allocate pointers to rows */
	 m=(double **)malloc((size_t)((nrow+NR_END)
					    *sizeof(double*)));
	 if (!m) nrerror("allocation failure in convert_matrix()");
	 m += NR_END;
	 m -= nrl;

	 /* set pointers to rows */
	 m[nrl]=a-ncl;
	 for(i=1,j=nrl+1;i<nrow;i++,j++) m[j]=m[j-1]+ncol;
	 /* return pointer to array of pointers to rows */
	 return m;
 }

 void free_vector(double *v, long nl, long nh)
 /* free a vector allocated with vector() */
 {
	 free((FREE_ARG) (v+nl-NR_END));
 }

 void free_fvector(float *v, long nl, long nh)
 /* free a float vector allocated with fvector() */
 {
	 free((FREE_ARG) (v+nl-NR_END));
 }

 void free_ivector(int *v, long nl, long nh)
 /* free an int vector allocated with ivector() */
 {
	 free((FREE_ARG) (v+nl-NR_END));
 }

 void free_cvector(unsigned char *v, long nl, long nh)
 /* free an unsigned char vector allocated with cvector() */
 {
	 free((FREE_ARG) (v+nl-NR_END));
 }

 void free_lvector(unsigned long *v, long nl, long nh)
 /* free an unsigned long vector allocated with lvector() */
 {
	 free((FREE_ARG) (v+nl-NR_END));
 }

 void free_dvector(double *v, long nl, long nh)
 /* free a double vector allocated with dvector() */
 {
	 free((FREE_ARG) (v+nl-NR_END));
 }

 void free_matrix(double  **m, long nrl, long nrh, long ncl, long nch)
 /* free a matrix allocated by matrix() */
 {
	 free((FREE_ARG) (m[nrl]+ncl-NR_END));
	 free((FREE_ARG) (m+nrl-NR_END));
 }

 void free_fmatrix(float **m, long nrl, long nrh, long ncl, long nch)
 /* free a float matrix allocated by fmatrix() */
 {
	 free((FREE_ARG) (m[nrl]+ncl-NR_END));
	 free((FREE_ARG) (m+nrl-NR_END));
 }

 void free_dmatrix(double **m, long nrl, long nrh, long ncl, long nch)
 /* free a double matrix allocated by dmatrix() */
 {
	 free((FREE_ARG) (m[nrl]+ncl-NR_END));
	 free((FREE_ARG) (m+nrl-NR_END));
 }

 void free_imatrix(int **m, long nrl, long nrh, long ncl, long nch)
 /* free an int matrix allocated by imatrix() */
 {
	 free((FREE_ARG) (m[nrl]+ncl-NR_END));
	 free((FREE_ARG) (m+nrl-NR_END));
 }

 void free_submatrix(float **b, long nrl, long nrh, long ncl, long nch)
 /* free a submatrix allocated by submatrix() */
 {
	 free((FREE_ARG) (b+nrl-NR_END));
 }

 void free_convert_matrix(float **b, long nrl, long nrh, long ncl, long nch)
 /* free a matrix allocated by convert_matrix() */
 {
	 free((FREE_ARG) (b+nrl-NR_END));
 }

 /***************************************************

   Matrix and Vector Printing routines

 */


 void print_vector( char *str, double *x, int n )
 {
   int     i;

   printf( "%s:\n", str );
   for( i = 1; i <= n; i++ )
     {
       if( (x[ i ] > 1) && (x[i] < 999) )
	 printf( " %3.1lf", x[ i ] );
       else
	 printf( " %2.2lg", x[ i ] );
     }
   printf( "\n" );
 }

 void print_quaternion( char *str, double *x )
 {
   int     i;

   printf( "%s:\n", str );
   for( i = 0; i < QUATERNION_SIZE; i++ )
     {
       if( (x[ i ] > 1) && (x[i] < 999) )
	 printf( " %3.1lf", x[ i ] );
       else
	 printf( " %2.2lg", x[ i ] );
     }
   printf( "\n" );
 }

 void print_matrix( char *str, double **A, int m, int n )
 {
   int     i, j;

   printf( "%s:  (%d x %d)\n", str, m, n );
   for( i = 1; i <= m; i++ )
     {
       printf( ">" );
       for( j = 1; j <= n; j++ )
	 {
	   printf( " %2.2lg", A[ i ][ j ] );
	 }
       printf( "\n" );
     }
 }

 double dmean(double *data, int n) {
     int i;
     double mean_val=0.0;
     for (i=0;i<n;i++) {
	 mean_val+=data[i];
     }
     mean_val/=n;
     return mean_val;
 }

 double dsigma(double *data, int n, double mean_val) {
     int i=0;
     double sigma_val=0;
     double temp_s=0;
     for (i=0;i<n;i++) {
	 sigma_val+=pow(data[i]-mean_val,2);
	 temp_s+=data[i]-mean_val;
     }
     temp_s=pow(temp_s,2)/n;
     sigma_val-=temp_s;
     sigma_val=sqrt(fabs(sigma_val)/(n-1));
     return sigma_val;
 }

 float fmean(float *data, int n) {
     int i;
     float mean_val=0.0;    
     for (i=0;i<n;i++) {
	 mean_val+=data[i];
     }
     mean_val/=n;
     return mean_val;
 }

 float fmean_clean(float *data, int n) {
     int i;
     float mean_val=0.0;    
     int nn=0;
     for (i=0;i<n;i++) {
       if (data[i]==(1*data[i])) {
	 mean_val+=data[i];
	 nn++;
       }
     }
     mean_val/=nn;
     return mean_val;
 }


 float fsigma(float *data, int n, float mean_val) {
     int i=0;
     float sigma_val=0;
     float temp_s=0;
     for (i=0;i<n;i++) {
	 sigma_val+=pow(data[i]-mean_val,2);
	 temp_s+=data[i]-mean_val;
     }
     temp_s=pow(temp_s,2)/n;
     sigma_val-=temp_s;
     sigma_val=sqrt(fabs(sigma_val)/(n-1));
     return sigma_val;
 }


 float fmedian(float *my_data, int n) {
   int i=0;
   int j=0;
   float temp;
   float median_val=0;
   int n_med=0;
   int n_max=n;
   float *data;

   data=(float *)malloc(n*sizeof(float));
   if (n_max>n) n_max=n;
   for (i=0;i<n_max;i++) {
     data[i]=my_data[i];
   }

   for (i=0;i<n_max;i++) {
     for (j=(i+1);j<n_max;j++) {
       if (data[j]>data[i]) {
	 temp=data[j];
	 data[j]=data[i];
	 data[i]=temp;
       }    
     }
   }
   n_med=(int) (n_max/2);
   median_val=data[n_med];
   free(data);
   return median_val;
 }


 double max(double a, double b) {
     double c;
     if (a>b) {
	 c=a;
     } else {
	 c=b;
     }
     return c;
 }

 int imax(int a, int b) {
     int c;
     if (a>b) {
	 c=a;
     } else {
	 c=b;
     }
     return c;
 }

 /*
  * This function returns the minimun power of 2 that excess
  * the input value
  */
 int min2power(int n){
   int m,j;
   m=0;
   j=1;
   while(m<=n) {
   //  while(m<n) {
     m=floor(pow(2,j));
     //    printf("%d, %d\m");
     j++;
   }
   return m;
 }



 /*
  * Simple image index, x,y conversions
  */
 //void index_xy(int x, int y, int n_x, int *index);
 //void xy_index(int *x, int *y, int n_x, int index);

 void index_xy(int x, int y, int n_x, int *index) {
   *index=x+y*n_x;
 }

 void xy_index(int *x, int *y, int n_x, int index) {
   float n_x_tmp,index_tmp;
   index_tmp=index;
   n_x_tmp=n_x;

   //  printf("%d %d %f %f \n",n_x,index,n_x_tmp,index_tmp);
   *x=index-n_x*floor(index/n_x);
   *y=((index-*x)/n_x);
 }

 /*
  * This function determine the values over the Fourier serie
  * considering angles in Degrees
  */

 void fourier_serie(float x, float afunc[], int ma) {
   int n,i,j;
   x=x/180*3.14159;
   for (j=1;j<=ma;j++) {
     if (j==1) {
       afunc[j]=1;
     } else {
       i=((int)(j/2))*2;
       if (i==j) {
	 afunc[j]=sin((j/2)*x);
       } else {
	 afunc[j]=cos(((j-1)/2)*x);
       }
     }
   }

 }


 /*
  * This function determine the values over the Polinom serie
  * considering angles in Degrees
  */

 void poly_serie(float x, float afunc[], int ma) {
   int j;
   for (j=1;j<=ma;j++) {
     afunc[j]=pow(x,j-1);
   }
 }

/* spline.c

	Copyright (c) Kapteyn Laboratorium Groningen 1990
	All Rights Reserved.

#>            spline.dc2

Purpose:      Spline interpolation routines.

Category:     MATH

File:         spline.c

Author:       P.R. Roelfsema

Description:

              Spline.c contains a number of routines relevant for
              spline interpolations, taken from "Numerical recipes
              in C".
              The following are available:

              SPLINE1  - 1D cbic spline interpolation.

Updates:      Jan 29, 1991: PRR, Creation date

Modified:     S.F.Sanchez 2005

#<

*/


#define OK		   0		/* no problems */
#define	NOMEMORY	  -1		/* no memory available for buffers */
#define EQALXES		  -2		/* two x-coordinates are equal */

#define true		   1
#define false		   0
#define MAXTXTLEN	  80

#define finit( fc , len ) { fc.a = malloc( ( len + 1 ) * sizeof( char ) ) ;  \
                            fc.l = len ; }

static int klo = -1 ;
static int khi = -1 ;


/*
    spline constructs a cubic spline given a set of x and y values, through
these values.

*/
static int spline( float *x   , float *y   , int  n   , 
                    float  yp1 , float  ypn , float *y2 )
{
	int  i,k;
	float p,qn,sig,un,*u;

	u=(float *)malloc((unsigned) (n-1)*sizeof(float));
        if (!u) return( NOMEMORY ) ;
	if (yp1 > 0.99e30)
		y2[0]=u[0]=0.0;
	else {
		y2[0] = -0.5;
		u[0]=(3.0/(x[1]-x[0]))*((y[1]-y[0])/(x[1]-x[0])-yp1);
	}
	for (i=1;i<=n-2;i++) {
		sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
		p=sig*y2[i-1]+2.0;
		y2[i]=(sig-1.0)/p;
		u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
		u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
	}
	if (ypn > 0.99e30)
		qn=un=0.0;
	else {
		qn=0.5;
		un=(3.0/(x[n-1]-x[n-2]))*(ypn-(y[n-1]-y[n-2])/(x[n-1]-x[n-2]));
	}
	y2[n-1]=(un-qn*u[n-2])/(qn*y2[n-2]+1.0);
	for (k=n-2;k>=0;k--)
		y2[k]=y2[k]*y2[k+1]+u[k];
	free(u);
        return( OK ) ;
}


/*
    splint uses the cubic spline generated with spline to interpolate values
in the XY  table.

*/
static int splint( float *xa , float *ya , float *y2a , 
                    int n    , float x   , float *y   )
{
        int  r   = 0 ;
	int  k;
	float h,b,a;

        if ( klo < 0 ){
  	   klo=0;
 	   khi=n-1;
        } else {
           if ( x < xa[klo] ) klo=0;
           if ( x > xa[khi] ) khi=n-1;
        }
	while (khi-klo > 1) {
		k=(khi+klo) >> 1;
		if (xa[k] > x) khi=k;
		else klo=k;
	}
	h=xa[khi]-xa[klo];
	if (h == 0.0) {
           setfblank_c( y ) ;
           r = EQALXES ;
        } else {
	   a=(xa[khi]-x)/h;
	   b=(x-xa[klo])/h;
	   *y=a*ya[klo]+b*ya[khi]+( (a*a*a-a)*y2a[klo]+
                                    (b*b*b-b)*y2a[khi] ) * (h*h) / 6.0;
        }
	return( r ) ;
}

/* 

#>            spline1.dc2

Function:     spline1

Purpose:      1D cubic spline interpolation.

Category:     MATH

File:         spline.c

Author:       P.R. Roelfsema

Use:          INTEGER SPLINE1( XI   ,               Input   real( > NIN )
                               YI   ,               Input   real( > NIN )
                               NIN  ,               Input   integer
                               XO   ,               Input   real( > NOUT )
                               YI   ,               Output  real( > NOUT )
                               NOUT ,               Output  integer

              SPLINE1 returns error codes:
                      >0 - number of undefined values in YI.
                       0 - no problems.
                      -1 - not enough memory to create internal tables.
                      -2 - the input array has two equal x-coordinates or
                           value in XO outside range of input coordinates >
                           interpolation problems.
              XI      Array containing input x-coordinates.
              YI      Array containing input y-coordinates.
              NIN     Number of (XI,YI) coordinate pairs.
              XO      Array containing x-coordinates for which y-coordinates
                      are to be interpolated.
              YO      Array containing interpolated y-coordinates.
                      If ALL YI are undefined, ALL YO are set to undefined.
              NOUT    Number of x-coordinates for which interpolation
                      is wanted.

Description:

		The interpolation is based on the cubic spline interpolation
	routines described in "Numerical recipes in C". For the interpolation
	data points in YI wich are undefined are skipped. Thus if undefined
	values are present, this routine will interpolate across them. 
	XI data must be in increasing order. The number of undefined values 
	in YI is returned.

Updates:      Jan 29, 1991: PRR, Creation date
              Aug  7, 1991: MV,  Reset of 'klo' in 'spline1_c'
                                 Documentation about XI.
              Aug  9, 1993: PRR, Set YO to undefined when all YI undefined.
#<

Fortran to C interface:

@ int function spline1( real , real , int , real , real , int ) 

*/

#define FBLANK 0x8

int spline1_c( float *xi , float *yi , int *nin  ,
                float *xo , float *yo , int *nout )
{
   float yp1 , ypn , *y2 , *xii , *yii , blank , alpha ;
   int  error , n , m , nblank , niin ;

   setfblank_c( &blank ) ;
   nblank = 0 ;
   for ( n = 0 ; n < *nin ; n++ )
      if ( yi[ n ] == blank ) nblank++ ;

   niin = *nin - nblank ;
   if ( nblank == 0 ) {
      xii = xi ;
      yii = yi ;
   } else if ( nblank == *nin ) {
      for ( n = 0 ; n < *nout ; n++ ) 
         yo[ n ] = blank ;
      return( nblank ) ;
   } else {
      xii = (float *)malloc( ( niin ) * sizeof( float ) ) ;
      yii = (float *)malloc( ( niin ) * sizeof( float ) ) ;
      if ( !xii || !yii ) return( NOMEMORY ) ;
      for ( n = 0 , m = 0 ; ( n < *nin ) && ( m < niin ) ; n++ ) {
         if ( yi[ n ] != blank ) {
            xii[ m ] = xi[ n ] ;
            yii[ m ] = yi[ n ] ;
            m += 1 ;
         }
      }
   }

   y2 = (float *)malloc( ( niin ) * sizeof( float ) ) ;
   if ( !y2 ) return( NOMEMORY ) ;

   alpha = 1.0 ;
   yp1 = 3e30 * alpha ;
   ypn = 3e30 * alpha ;

   error = spline( xii , yii , niin , yp1 , ypn , y2 ) ;
   if ( error < 0 ) return( error ) ;
   
   klo = -1; /* Reset 'klo' before table interpolation */
   for ( n = 0  ; n < *nout ; n++ ) {
      error = splint( xii , yii , y2 , niin , xo[ n ] , &yo[ n ] ) ;
      if ( error < 0 ) return( error ) ;
   }

   if ( nblank != 0 ) {
      free( xii ) ;
      free( yii ) ;
   }
   free( y2  ) ;

   return( nblank ) ;
}

void	setfblank_c( unsigned int *datum )
{
   *datum = FBLANK;				/* this is it */
}


int spline1_old( float *xi , float *yi , int nin  ,
                float *xo , float *yo , int nout )
{
   float *y2;
   int i;
   y2 = (float *)malloc(nin*sizeof(float));  
//   printf("N=[%d,%d]",nin,nout);
   spline(xi,yi,nin,3e30,3e30,y2);
   for (i=0;i<nout;i++) {             
       splint(xi,yi,y2,nin,xo[i],&yo[i]);
//       printf("PASO %d %f %f \n",i,xo[i],yo[i]);
   }
   free(y2);
   return(0);
}


/*
		    coeff = (float *)malloc((npoly)*sizeof(float));  	
		    polyfit(peaks_cen,peaks_now,n_peak,coeff,npoly);
		    for (i=0;i<image.nx;i++) {
			a_dist[i+j*image.nx]=0;
			for (kk=0;kk<npoly;kk++) {
			//printf("%d %f\n",kk,coeff[kk]);
			    a_dist[i+j*image.nx]=a_dist[i+j*image.nx]+coeff[kk]*pow(i,kk);
			}	    
			a_i_new[i]=a_dist[i+j*image.nx];
			a_i_old[i]=i;
			da_i_new[i]=-a_i_new[i]+a_i_old[i];
		    }	
		    free(coeff);
*/
// Uses POLYINT
int spline1_polint( float *xi , float *yi , int nin  ,
                float *xo , float *yo , int nout )
{
   float error_y;
   int i;
   for (i=0;i<nout;i++) {  
       fflush(stdout);
       POLINT(xi,yi,nin,xo[i],&yo[i],&error_y);
   }
   return(0);
}

// Uses polyfit
int spline1( float *xi , float *yi , int nin  ,
                float *xo , float *yo , int nout )
{
   float error_y;
   int i,j,k,I,NS,i_min,i_max;
   float DIF,DIFT; 
   float xt[12],yt[12],coeff[12];
   int npoly=2;
//   int delta_i=1;
   int range_i=2;
//   float *y2;

//   range_i=2*delta_i+1;
   for (i=0;i<nout;i++) {  
       // We look for the nearest point.
       NS=0;
       DIF = fabs(xo[i]-xi[0]);
       for (I=0; I<nin; I++) {
	   DIFT = fabs(xo[i]-xi[I]);
	   if (DIFT < DIF) {
	       NS=I;               //index of closest table entry
	       DIF = DIFT;
	   }
       }
       if (DIF==0) {
	   yo[i]=yi[NS];
       } else {
	   if (xo[i]>xi[NS]) {
	       i_min=NS;
	       i_max=NS+1;	       
	   } else {
	       i_min=NS-1;
	       i_max=NS;	       
	   }

	   if (i_min<0) {	       
	       i_min=0;
	       i_max=range_i-1;
	   }
	   if (i_max>=nout) {	       
	       i_min=nout-range_i;
	       i_max=nout-1;
	   }
	   k=0;
	   for (j=i_min;j<=i_max;j++) {
	       xt[k]=xi[j];
	       yt[k]=yi[j];
	       k++;
	   }
/*
	   y2 = (float *)malloc(range_i*sizeof(float));  
	   spline(xt,yt,range_i,3e30,3e30,y2);
	   splint(xt,yt,y2,range_i,xo[i],&yo[i]);
	   free(y2);
*/

	   polyfit(xt,yt,range_i,coeff,npoly);	   
	   yo[i]=0;
	   for (k=0;k<npoly;k++) {
	       yo[i]=yo[i]+coeff[k]*pow(xo[i],k);
	   }	    
   
       }


   }
   return(0);
}

#define  NMAX  25

void POLINT(float *XA, float *YA, int N, float X, float *Y, float *DY) {
/****************************************************
*     Polynomial Interpolation or Extrapolation     *
*            of a Discreet Function                 *
* ------------------------------------------------- *
* INPUTS:                                           *
*   XA:    Table of abscissas (N)                   *
*   YA:    Table of ordinates (N)                   *
*    N:    Number of points                         *
*    X:    Interpolation abscissa value             *
* OUTPUTS:                                          *
*    Y:    Returned estimation of function for X    *
*   DY:    Estimated error for Y                    *
****************************************************/
//  float C[NMAX], D[NMAX];//
//    float *C,*D;
  float DEN,DIF,DIFT,HO,HP,W;
  int I,M,NS;
  float *C, *D;


  C = (float *)malloc(N*sizeof(float));  
  D = (float *)malloc(N*sizeof(float));  
//  printf("%f %f %d %f\n",XA[0],YA[0],N,X);


  NS=0;
  DIF = fabs(X-XA[0]);
  for (I=0; I<N; I++) {
    DIFT = fabs(X-XA[I]);
    if (DIFT < DIF) {
      NS=I;               //index of closest table entry
      DIF = DIFT;
    }
    C[I]=YA[I];           //Initialize the C's and D's
    D[I]=YA[I];
  }
  *Y=YA[NS];              //Initial approximation of Y
//  printf("Y=%f\n",Y);
  NS--;
  for (M=1; M<N; M++) {
      //     printf("%d %d\n",M,N-1);
	for (I=0; I<N-M; I++) {
	    HO=XA[I]-X;
	    HP=XA[I+M]-X;
	    W=C[I+1]-D[I];
	    DEN=HO-HP;
	    if (DEN == 0.0) {
		printf("\n Error: two identical abscissas.\n\n");
		return;
	    }
	    DEN=W/DEN;
	    D[I]=HP*DEN;         //Update the C's and D's
	    C[I]=HO*DEN;
	}
	if (2*NS < N-M)        //After each column in the tableau XA is completed,
	    *DY=C[NS+1];         //we decide which correction, C or D, we want to   
	else {                 //add to our accumulating value of Y, i.e. which   
	    *DY=D[NS];           //path to take through the tableau, for (king up or
	    NS--;                //down. We do this in such a way as to take the    
	}                      //most "straight line" route through the tableau to
	*Y = *Y + *DY;         //its apex, updating NS accordingly to keep track  
	                       //of where we are. This route keeps the partial    
  }                        //approximations centered (insofar as possible) on 
                           //the target X.The last DY added is thus the error 
}                          //indication.



